package org.jboss.docs.fd.ejb.manager;

import java.rmi.RemoteException;
import java.util.Collection;
import java.util.Set;
import java.util.Iterator;
import javax.ejb.SessionBean;
import javax.ejb.SessionContext;
import javax.ejb.EJBException;
import javax.ejb.CreateException;
import javax.ejb.FinderException;
import javax.ejb.RemoveException;
import javax.naming.NamingException;
import javax.naming.InitialContext;
import javax.naming.Context;

import org.jboss.docs.fd.ejb.student.StudentLocalHome;
import org.jboss.docs.fd.ejb.student.StudentLocal;
import org.jboss.docs.fd.ejb.teacher.TeacherLocalHome;
import org.jboss.docs.fd.ejb.teacher.TeacherLocal;
import org.jboss.docs.fd.ejb.group.GroupLocalHome;
import org.jboss.docs.fd.ejb.group.GroupLocal;
import org.jboss.docs.fd.ejb.group.GroupPK;

/**
 * ManagerSessionEJB.
 *
 * @ejb.bean
 *    type="Stateless"
 *    name="ManagerSession"
 *    jndi-name="ManagerSession"
 *    generate="true"
 *    view-type="remote"
 *
 * @ejb.transaction type="Required"
 *
 * @ejb.util generate="physical"
 *
 * @ejb.ejb-ref ejb-name="Student" view-type="local"
 * @weblogic.ejb-local-reference-description
 *    ejb-ref-name="ejb/Student"
 *    jndi-name="fd/sample/Student"
 *
 * @ejb.ejb-ref ejb-name="Teacher" view-type="local"
 * @weblogic.ejb-local-reference-description
 *    ejb-ref-name="ejb/Teacher"
 *    jndi-name="fd/sample/Teacher"
 *
 * @ejb.ejb-ref ejb-name="Group" view-type="local"
 * @weblogic.ejb-local-reference-description
 *    ejb-ref-name="ejb/Group"
 *    jndi-name="fd/sample/Group"
 *
 * @author <a href="mailto:aloubyansky@hotmail.com">Alex Loubyansky</a>
 */
public class ManagerSessionEJB
   implements SessionBean
{
   // Constants -------------------------------------------------------
   private static final String STUDENT_COMP = "java:/comp/env/ejb/Student";
   private static final String TEACHER_COMP = "java:/comp/env/ejb/Teacher";
   private static final String GROUP_COMP = "java:/comp/env/ejb/Group";

   // Attributes ------------------------------------------------------
   private StudentLocalHome studentLocalHome;
   private TeacherLocalHome teacherLocalHome;
   private GroupLocalHome groupLocalHome;

   // Business methods ------------------------------------------------
   /**
    * @ejb.interface-method
    */
   public void createTeacherInfo(String name)
   {
      TeacherLocalHome teacherLocalHome = getTeacherLocalHome();
      try
      {
         teacherLocalHome.create( name ).getTeacherId();
      }
      catch( CreateException e )
      {
         throw new EJBException( "Couldn't create teacher info: ", e );
      }
   }

   /**
    * @ejb.interface-method
    */
   public void createGroupInfo( String deptId, Long groupNo, String description )
   {
      GroupLocalHome groupLocalHome = getGroupLocalHome();
      try
      {
         groupLocalHome.create( deptId, groupNo, description );
      }
      catch( CreateException ce )
      {
         throw new EJBException( "Couldn't create group info: ", ce );
      }
   }

   /**
    * @ejb.interface-method
    */
   public void createStudentInfo( String name, String deptId, Long groupNo )
   {
      studentLocalHome = getStudentLocalHome();
      groupLocalHome = getGroupLocalHome();
      try
      {
         GroupLocal group = groupLocalHome.findByPrimaryKey(
            new GroupPK( deptId, groupNo ) );
         studentLocalHome.create( name, group );
      }
      catch( FinderException fe )
      {
         throw new EJBException( "Group " + deptId + groupNo + " couldn't be found." );
      }
      catch( CreateException e )
      {
         throw new EJBException( "Couldn't create student info: ", e );
      }
   }

   /**
    * @ejb.interface-method
    */
   public void addGroupToTeacher( String teacherName, String deptId, Long groupNo )
   {
      teacherLocalHome = getTeacherLocalHome();
      groupLocalHome = getGroupLocalHome();

      try
      {
         Collection teachers = teacherLocalHome.findByName( teacherName );
         if( teachers.isEmpty() )
            throw new EJBException( "Teacher with name " + teacherName + " not found." );
         TeacherLocal teacher = (TeacherLocal)teachers.iterator().next();
         GroupLocal group = groupLocalHome.
            findByPrimaryKey( new GroupPK( deptId, groupNo ) );
         teacher.getGroups().add( group );
      }
      catch( FinderException fe )
      {
         throw new EJBException( "Coudn't find entity: ", fe );
      }
      catch( Exception e )
      {
         throw new EJBException( "Couldn't add group " + deptId + groupNo
            + " to teacher " + teacherName );
      }
   }

   /**
    * @ejb.interface-method
    */
   public Set getStudentsForTeacher( String teacherName )
   {
      teacherLocalHome = getTeacherLocalHome();
      try
      {
         return teacherLocalHome.getStudentsForTeacher( teacherName );
      }
      catch( FinderException fe )
      {
         throw new EJBException( "Couldn't fetch students for " + teacherName );
      }
   }

   /**
    * @ejb.interface-method
    */
   public void cleanDataSource()
   {
      Iterator iter;

      try
      {
         // remove teachers
         iter = getTeacherLocalHome().findAll().iterator();
         while( iter.hasNext() )
            ((TeacherLocal) iter.next()).remove();
      }
      catch( FinderException fe ) { }
      catch( RemoveException re ) { }

      try
      {
         // remove students
         iter = getStudentLocalHome().findAll().iterator();
         while( iter.hasNext() )
            ((StudentLocal) iter.next()).remove();
      }
      catch( FinderException fe ) { }
      catch( RemoveException re ) { }

      try
      {
         // remove groups
         iter = getGroupLocalHome().findAll().iterator();
         while( iter.hasNext() )
            ((GroupLocal) iter.next()).remove();
      }
      catch( FinderException fe ) { }
      catch( RemoveException re ) { }
   }

   // SessionBean implementation --------------------------------------
   /**
    * @ejb.create-method
    */
   public void ejbCreate() throws CreateException { }

   public void setSessionContext( SessionContext sc ) { }

   public void ejbActivate() {}
   public void ejbPassivate() {}
   public void ejbRemove() {}

   // Private ---------------------------------------------------------
   private StudentLocalHome getStudentLocalHome()
   {
      if( studentLocalHome == null )
      {
         try
         {
            InitialContext ic = new InitialContext();
            studentLocalHome = (StudentLocalHome) ic.lookup( STUDENT_COMP );
         }
         catch( NamingException ne )
         {
            throw new EJBException( "Couldn't find StudentLocalHome with " + STUDENT_COMP );
         }
      }
      return studentLocalHome;
   }

   private TeacherLocalHome getTeacherLocalHome()
   {
      if( teacherLocalHome == null )
      {
         try
         {
            InitialContext ic = new InitialContext();
            teacherLocalHome = (TeacherLocalHome) ic.lookup( TEACHER_COMP );
         }
         catch( NamingException ne )
         {
            throw new EJBException( "Couldn't find TeacherLocalHome with " + TEACHER_COMP );
         }
      }
      return teacherLocalHome;
   }

   private GroupLocalHome getGroupLocalHome()
   {
      if( groupLocalHome == null )
      {
         try
         {
            InitialContext ic = new InitialContext();
            groupLocalHome = (GroupLocalHome) ic.lookup( GROUP_COMP );
         }
         catch( NamingException ne )
         {
            throw new EJBException( "Couldn't find GroupLocalHome with " + GROUP_COMP );
         }
      }
      return groupLocalHome;
   }
}
