package org.jboss.docs.fd.ejb.sequence;

import java.rmi.RemoteException;
import javax.ejb.SessionBean;
import javax.ejb.SessionContext;
import javax.ejb.EJBException;
import javax.ejb.CreateException;
import javax.ejb.FinderException;
import javax.ejb.TransactionRolledbackLocalException;
import javax.naming.NamingException;
import java.util.Hashtable;

/**
 * SequenceSessionEJB.
 *
 * @ejb.bean
 *    type="Stateless"
 *    name="SequenceSession"
 *    local-jndi-name="fd/sample/SequenceSession"
 *    generate="true"
 *    view-type="local"
 *
 * @ejb.transaction type="Required"
 *
 * @ejb.util generate="physical"
 *
 * @ejb.ejb-ref
 *    ejb-name="Sequence"
 *    view-type="local"
 *    ref-name="Sequence"
 *
 * @ejb.env-entry
 *    name="retryCount"
 *    type="java.lang.Integer"
 *    value="5"
 * @ejb.env-entry
 *    name="blockSize"
 *    type="java.lang.Integer"
 *    value="10"
 *
 * @author <a href="mailto:aloubyansky@hotmail.com">Alex Loubyansky</a>
 */
public class SequenceSessionEJB
   implements SessionBean
{
   // Constants -------------------------------------------------------
   private static final String SEQUENCE_COMP = "java:/comp/env/ejb/Sequence";

   // Attributes ------------------------------------------------------
   private Hashtable entries = new Hashtable();
   private int blockSize;
   private int retryCount;
   private SequenceLocalHome sequenceHome;

   // Business methods ------------------------------------------------
   /**
    * @ejb.interface-method
    */
   public int getNextSequenceNumber(String name)
   {
      try
      {
         Entry entry = (Entry) entries.get(name);

         if (entry == null)
         {
            // add an entry to the sequence table
            entry = new Entry();
            try
            {
               entry.sequence = sequenceHome.findByPrimaryKey(name);
            }
            catch (FinderException e)
            {
               // if not found then create it
               entry.sequence = sequenceHome.create(name);
            }

            entries.put(name, entry);
         }

         if (entry.last % blockSize == 0)
         {
            for (int retry = 0; true; retry++)
            {
               try
               {
                  entry.last = entry.sequence.getValueAfterIncrementingBy(blockSize);
                  break;
               }
               catch (TransactionRolledbackLocalException e)
               {
                  if (retry < retryCount)
                  {
                     // hit concurrency exception, try again
                     continue;
                  }
                  else
                  {
                     // tried too many times, fail
                     throw new javax.ejb.EJBException(e);
                  }
               }
            }
         }

         return entry.last++;
      }
      catch (CreateException e)
      {
         throw new EJBException(e);
      }
   }

   // SessionBean implementation --------------------------------------
   /**
    * @ejb.create-method
    */
    public void ejbCreate() throws CreateException { }

    public void setSessionContext(final SessionContext sc)
    {
       try
       {
          javax.naming.Context namingContext = new javax.naming.InitialContext();
          blockSize = ((Integer) namingContext.lookup("java:comp/env/blockSize")).intValue();
          retryCount = ((Integer) namingContext.lookup("java:comp/env/retryCount")).intValue();
          sequenceHome = (SequenceLocalHome)namingContext.lookup( SEQUENCE_COMP );
       }
       catch(NamingException e)
       {
          throw new EJBException(e);
       }
    }

   public void ejbActivate() {}
   public void ejbPassivate() {}
   public void ejbRemove() {}

   // Inner classes -----------------------------------------------------
   private class Entry
   {
      SequenceLocal sequence;
      int last;
   }
}
