package org.jboss.docs.fd.ejb.student;

import org.jboss.docs.fd.ejb.sequence.SequenceSessionLocalHome;
import org.jboss.docs.fd.ejb.sequence.SequenceSessionLocal;
import org.jboss.docs.fd.ejb.group.GroupLocal;

import javax.ejb.EntityBean;
import javax.ejb.CreateException;
import javax.ejb.EntityContext;
import javax.ejb.RemoveException;
import javax.ejb.EJBException;
import javax.ejb.FinderException;
import javax.naming.NamingException;
import javax.naming.InitialContext;
import java.util.Iterator;

/**
 * Local entity bean models a student data.
 *
 * @ejb.bean
 *    name="Student"
 *    display-name="Student EJB"
 *    generate="true"
 *    local-jndi-name="fd/sample/Student"
 *    view-type="local"
 *    type="CMP"
 *    reentrant="False"
 *    cmp-version="2.x"
 *    primkey-field="studentId"
 *
 * @ejb.pk class="java.lang.Long" generate="false"
 *
 * @ejb.finder
 *    view-type="local"
 *    signature="Collection findAll()"
 *    query="SELECT OBJECT(s) FROM Student AS s"
 *
 * @ejb.transaction type="Required"
 *
 * @ejb.ejb-ref ejb-name="SequenceSession" view-type="local"
 * @weblogic.ejb-local-reference-description
 *    ejb-ref-name="ejb/SequenceSession"
 *    jndi-name="fd/sample/SequenceSession"
 *
 * @ejb.env-entry
 *    name="sequenceName"
 *    type="java.lang.String"
 *    value="student_sequence"
 *
 * @ejb.persistence table-name="students"
 *
 * @jboss.create-table "${jboss.create.table}"
 * @jboss.remove-table "${jboss.remove.table}"
 *
 * @author <a href="mailto:aloubyansky@hotmail.com">Alex Loubyansky</a>
 */
public abstract class StudentEJB
   implements EntityBean
{
   // Constants ----------------------------------------------------------
   private static final String SEQUENCE_SESSION_COMP = "java:/comp/env/ejb/SequenceSession";

   // Attributes ---------------------------------------------------------
   private EntityContext ctx;

   /** the sequence name for this bean */
   private String sequenceName;

   /** cache sequence session home */
   private SequenceSessionLocalHome seqSessionHome;

   // CMP accessors ------------------------------------------------------
   /**
    * Student id: primary key field
    *
    * @ejb.persistent-field
    * @ejb.pk-field
    *
    * @ejb.persistence
    *    column-name="student_id"
    *    jdbc-type="INTEGER"
    *    sql-type="INTEGER(5)"
    *
    * @ejb.interface-method
    */
   public abstract Long getStudentId();
	public abstract void setStudentId(java.lang.Long studentId);

   /**
    * Student name: persistent field
    *
    * @ejb.persistent-field
    *
    * @ejb.persistence
    *    column-name="name"
    *    jdbc-type="VARCHAR"
    *    sql-type="VARCHAR(50)"
    *
    * @ejb.interface-method
    */
   public abstract String getName();
   /**
    * @ejb.interface-method
    */
   public abstract void setName(String name);

   // CMR accessors -------------------------------------------------------

   /**
    * Student - Group: many-to-one bidirectional
    *
    * @ejb.interface-method
    *
    * @ejb.relation
    *    name="Student-Group"
    *    role-name="Student-belongsto-Group"
    *    cascade-delete="yes"
    *
    * @jboss.relation
    *    fk-constraint="false"
    *    related-pk-field="departmentId"
    *    fk-column="dept_id_fk"
    * @jboss.relation
    *    fk-constraint="false"
    *    related-pk-field="groupNo"
    *    fk-column="group_no_fk"
    *
    * @weblogic.column-map
    *    foreign-key-column="group_no_fk"
    *    key-column="group_no"
    * @weblogic.column-map
    *    foreign-key-column="dept_id_fk"
    *    key-column="dept_id"
    */
   public abstract GroupLocal getGroup();
   /**
    * @ejb.interface-method
    */
   public abstract void setGroup( GroupLocal group );

   // EntityBean implementation -------------------------------------------
   /**
    * @ejb.create-method
    */
   public Long ejbCreate(String name)
      throws CreateException
	{
      int nextIndex;
      try
      {
         nextIndex = getNextSequenceNumber();
      }
      catch(Exception e)
      {
         throw new CreateException( "Couldn't get next sequence index for student: " + e );
      }

      this.setStudentId( new Long(nextIndex) );
		this.setName(name);
		return null;
	}

   /**
    * @ejb.create-method
    */
   public Long ejbCreate(String name, GroupLocal group)
      throws CreateException
	{
      int nextIndex;
      try
      {
         nextIndex = getNextSequenceNumber();
      }
      catch(Exception e)
      {
         throw new CreateException( "Couldn't get next sequence index for student: " + e );
      }

      this.setStudentId( new Long(nextIndex) );
		this.setName(name);
		return null;
	}

   public void ejbPostCreate(String name) {}

   public void ejbPostCreate(String name, GroupLocal group)
   {
      setGroup( group );
   }

   public void setEntityContext(EntityContext ctx)
   {
      this.ctx = ctx;
   }

	public void unsetEntityContext() {}

	public void ejbActivate() {}
	public void ejbLoad() {}
	public void ejbPassivate() {}
	public void ejbRemove() throws RemoveException {}
	public void ejbStore() {}

   // Private methods --------------------------------------------------
   private int getNextSequenceNumber()
      throws Exception
   {
      int nextIndex;
      try
      {
         SequenceSessionLocalHome seqHome = getSequenceManagerLocalHome();
         SequenceSessionLocal seqLocal = seqHome.create();
         nextIndex = seqLocal.getNextSequenceNumber( getSequenceName() );
      }
      catch(Exception e)
      {
         throw e;
      }
      return nextIndex;
   }

   private SequenceSessionLocalHome getSequenceManagerLocalHome()
      throws Exception
   {
      if(seqSessionHome != null) return seqSessionHome;
      try
      {
         InitialContext ic = new InitialContext();
         seqSessionHome = (SequenceSessionLocalHome)
            ic.lookup( SEQUENCE_SESSION_COMP );
      }
      catch( Exception e )
      {
         throw e;
      }
      return seqSessionHome;
   }

   private String getSequenceName()
   {
      if( sequenceName != null ) return sequenceName;
      try
      {
         InitialContext ic = new InitialContext();
         sequenceName = (String) ic.lookup("java:comp/env/sequenceName");
      }
      catch(NamingException ne)
      {
         sequenceName = this.getClass().getName();
      }
      return sequenceName;
   }
}
