package org.jboss.docs.fd.ejb.teacher;

import org.jboss.docs.fd.ejb.sequence.SequenceSessionLocalHome;
import org.jboss.docs.fd.ejb.sequence.SequenceSessionLocal;

import javax.ejb.EntityBean;
import javax.ejb.CreateException;
import javax.ejb.EntityContext;
import javax.ejb.RemoveException;
import javax.ejb.EJBException;
import javax.ejb.FinderException;
import javax.naming.NamingException;
import javax.naming.InitialContext;
import java.util.Collection;
import java.util.Set;
import java.util.Iterator;

/**
 * Local entity bean models teacher's data.
 *
 * @ejb.bean
 *    name="Teacher"
 *    display-name="Teacher EJB"
 *    generate="true"
 *    local-jndi-name="fd/sample/Teacher"
 *    view-type="local"
 *    type="CMP"
 *    reentrant="False"
 *    cmp-version="2.x"
 *    primkey-field="teacherId"
 *
 * @ejb.pk class="java.lang.Long" generate="false"
 *
 * @ejb.finder
 *    view-type="local"
 *    signature="Collection findAll()"
 *    query="SELECT OBJECT(t) FROM Teacher AS t"
 *
 * @ejb.finder
 *    view-type="local"
 *    signature="Collection findByName(java.lang.String name)"
 *    query="SELECT OBJECT(t) FROM Teacher AS t WHERE (t.name = ?1)"
 *
 * @ejb.transaction type="Required"
 *
 * @ejb.ejb-ref ejb-name="SequenceSession" view-type="local"
 * @weblogic.ejb-local-reference-description
 *    ejb-ref-name="ejb/SequenceSession"
 *    jndi-name="fd/sample/SequenceSession"
 *
 * @ejb.env-entry
 *    name="sequenceName"
 *    type="java.lang.String"
 *    value="teacher_sequence"
 *
 * @ejb.persistence table-name="teachers"
 *
 * @jboss.create-table "${jboss.create.table}"
 * @jboss.remove-table "${jboss.remove.table}"
 *
 * @author <a href="mailto:aloubyansky@hotmail.com">Alex Loubyansky</a>
 */
public abstract class TeacherEJB
   implements EntityBean
{
   // Attributes ---------------------------------------------------------
   private static final String SEQUENCE_SESSION_COMP = "java:/comp/env/ejb/SequenceSession";
   private EntityContext ctx;

   /** the sequence name for this bean */
   private String sequenceName;

   /** cache sequence session home */
   private SequenceSessionLocalHome seqSessionHome;

   // CMP accessors ------------------------------------------------------
   /**
    * Teacher id: primary key field
    *
    * @ejb.persistent-field
    * @ejb.pk-field
    *
    * @ejb.persistence
    *    column-name="teacher_id"
    *    jdbc-type="INTEGER"
    *    sql-type="INTEGER(5)"
    *
    * @ejb.interface-method
    */
   public abstract Long getTeacherId();
	public abstract void setTeacherId(java.lang.Long teacherId);

   /**
    * Teacher name: persistent field
    *
    * @ejb.persistent-field
    *
    * @ejb.persistence
    *    column-name="name"
    *    jdbc-type="VARCHAR"
    *    sql-type="VARCHAR(50)"
    *
    * @ejb.interface-method
    */
   public abstract String getName();
   /**
    * @ejb.interface-method
    */
   public abstract void setName(String name);

   // CMR

   /**
    * Teacher - Group: many-to-many bidirectional
    *
    * @ejb.interface-method
    *
    * @ejb.relation
    *    name="Teacher-Group"
    *    role-name="Teacher-has-Groups"
    *
    * @jboss.relation-table
    *    table-name="teacher_group"
    *    create-table="${jboss.create.table}"
    *    remove-table="${jboss.remove.table}"
    * @jboss.relation
    *    fk-constraint="false"
    *    related-pk-field="departmentId"
    *    fk-column="dept_id_fk"
    * @jboss.relation
    *    fk-constraint="false"
    *    related-pk-field="groupNo"
    *    fk-column="group_no_fk"
    *
    * @weblogic.relation join-table-name="teacher_group"
    * @weblogic.column-map
    *    foreign-key-column="teacher_id_fk"
    *    key-column="teacher_id"
    */
   public abstract Collection getGroups();
   /**
    * @ejb.interface-method
    */
   public abstract void setGroups( Collection groups );

   // Finders/Selectors ---------------------------------------------------
   /**
    * @ejb.select
    *    signature="java.util.Set ejbSelectStudents(java.lang.String teacherName)"
    *    query="SELECT DISTINCT s.name FROM Teacher AS t, IN(t.groups) g, IN(g.students) s WHERE (t.name = ?1)"
    */
   public abstract Set ejbSelectStudents( String teacherName )
      throws FinderException;

   // Home methods --------------------------------------------------------
   /**
    * @ejb.home-method
    */
   public Set ejbHomeGetStudentsForTeacher( String teacherName )
      throws FinderException
   {
      return ejbSelectStudents( teacherName );
   }

   // EntityBean implementation -------------------------------------------
   /**
    * @ejb.create-method
    */
   public Long ejbCreate(String name)
      throws CreateException
	{
      int nextIndex;
      try
      {
         nextIndex = getNextSequenceNumber();
      }
      catch(Exception e)
      {
         throw new CreateException( "Couldn't get next sequence index for teacher: " + e );
      }

      this.setTeacherId( new Long(nextIndex) );
		this.setName(name);
		return null;
	}

   public void ejbPostCreate(String name) {}

   public void setEntityContext(EntityContext ctx)
   {
      this.ctx = ctx;
   }

	public void unsetEntityContext() {}

	public void ejbActivate() {}
	public void ejbLoad() {}
	public void ejbPassivate() {}
	public void ejbRemove() throws RemoveException {}
	public void ejbStore() {}

   // Private methods --------------------------------------------------
   private int getNextSequenceNumber()
      throws Exception
   {
      int nextIndex;
      try
      {
         SequenceSessionLocalHome seqHome = getSequenceManagerLocalHome();
         SequenceSessionLocal seqLocal = seqHome.create();
         nextIndex = seqLocal.getNextSequenceNumber( getSequenceName() );
      }
      catch(Exception e)
      {
         throw e;
      }
      return nextIndex;
   }

   private SequenceSessionLocalHome getSequenceManagerLocalHome()
      throws Exception
   {
      if(seqSessionHome != null) return seqSessionHome;
      try
      {
         InitialContext ic = new InitialContext();
         seqSessionHome = (SequenceSessionLocalHome)
            ic.lookup( SEQUENCE_SESSION_COMP );
      }
      catch( Exception e )
      {
         throw e;
      }
      return seqSessionHome;
   }

   private String getSequenceName()
   {
      if( sequenceName != null ) return sequenceName;
      try
      {
         InitialContext ic = new InitialContext();
         sequenceName = (String) ic.lookup("java:comp/env/sequenceName");
      }
      catch(NamingException ne)
      {
         sequenceName = this.getClass().getName();
      }
      return sequenceName;
   }
}
